
(** librsvg bindings *)

(** {3 Low-level API} *)

type t

(** Create a new SVG loader. The first argument sets the size of the
   rendered surface. *)
val make : (int -> int -> int * int) -> t

(* val reset : t -> unit *)
(* val set_default_dpi : float -> unit *)
(* val set_dpi : t -> float -> unit *)

(** Feed data from a string buffer to the SVG loader *)
val write : t -> string -> off:int -> len:int -> unit

(** Close the loader *)
val close : t -> unit

(** Read an entire channel. The loader is closed at the end.*)
val write_channel : t -> in_channel -> unit

(** Get the rendered picture as a SDL surface.*)
val get_surface : t -> Sdlvideo.surface

(** {3 Convenience API} *)

val from_file : string -> Sdlvideo.surface
val from_file_at_zoom :
  string -> x_zoom:float -> y_zoom:float -> Sdlvideo.surface
val from_file_at_size :
  string -> width:int -> height:int -> Sdlvideo.surface
val from_file_at_max_size :
  string -> max_width:int -> max_height:int -> Sdlvideo.surface
val from_file_at_zoom_with_max :
  string ->
  x_zoom:float ->
  y_zoom:float -> max_width:int -> max_height:int -> Sdlvideo.surface
