(* $Id: example.ml,v 1.9 2002/03/26 16:30:49 xtrm Exp $ *)

open Sdl
open Sdlvideo
open Sdlevent

let () = Sdl.init ~auto_clean:true [`EVERYTHING]

let screen = Sdlvideo.set_video_mode ~w:640 ~h:480 []

let get_surface s =
  let f w h = 
    Printf.printf "Size callback, w=%d, h=%d\n" w h; flush stdout;
    131, 32
  let handle = Rsvg.make f in
  let fd = open_in s in
  let size = in_channel_length fd in
  let buf = String.create size in
  let read = really_input fd buf 0 size in
  Rsvg.write handle buf 0 size ;
  Rsvg.close handle ;
  let surface = Rsvg.get_surface handle in
  close_in fd ;
  surface

let surface = get_surface "tetris1.svg"
(*
let () = fill_rect screen (map_RGB screen red)
let () = put_pixel screen ~x:160 ~y:100 blue
*)

(*
let () = flip screen



let () = pump ()
let () = quit()
*)
(*
let () = Sdlmouse.show_cursor false

let font = try Sdlttf.open_font "Arial.ttf" 20 with
  | Sdlttf.SDLttf_exception s ->
    let cwd = Unix.getcwd () in 
    Printf.eprintf "Unable to open font %s/Arial.ttf\n" cwd; exit 1

module Sprite = struct
  let black = (0,0,0)
  let white = (255,255,255)
  let max_message_length = 20
  let write_message screen x y m =
    let l = String.length m in
    let s = 
      if l < max_message_length then
        m ^ (String.make (20-l) ' ') 
      else if l > max_message_length then
        String.sub m 0 20
      else m in
    let text = Sdlttf.render_text font (Sdlttf.SOLID white) s in
    let w,h,p = surface_dims text in
    let r = rect x y w h in
    fill_rect ~rect:r screen (map_RGB screen black);
    blit_surface ~src:text ~dst:screen ~dst_rect:r ();
    update_rect ~rect:r screen;
    ()
  type path = (int * int) list
  type pos = { x:int; y:int; w:int; h:int }
  let path =
  [10,10;0,10;10,10;0,10;10,10;10,-10;0,-10;-10,-10;-10,10;0,10;-10,10;-10,10]
  let sprite = Sdlttf.render_text font (Sdlttf.SOLID white) "X"
  let p2r p = rect p.x p.y p.w p.h
  let clip_rect screen sprite pos =
    let w,h,p = surface_dims sprite in
    let sw,sh,sp = surface_dims screen in
    let mx = sw - w in
    let my = sh - h in
    let y = if pos.y > my then my else if pos.y < 0 then 0 else pos.y in
    let x = if pos.x > mx then mx else if pos.x < 0 then 0 else pos.x in
    {pos with x = x; y = y}
  let move_sprite screen opos npos sprite =
    let orect = p2r (clip_rect screen sprite opos) in
    let nrect = p2r (clip_rect screen sprite npos) in
    fill_rect ~rect:orect screen (map_RGB screen black);
    blit_surface ~src:sprite ~dst:screen ~dst_rect:nrect ();
    update_rects [orect; nrect] screen;
    ()
  let follow_path screen sprite path current pos =
    ()    
  let pos_ref =
    let w,h,p = surface_dims sprite in
    ref { x=50; y=50; w=w; h=h; }
  let path_ref = ref path
  let idle_loop screen =
    Sdltimer.delay 200;
    let opos = !pos_ref in
    let dx,dy = match !path_ref with 
      | [] -> path_ref := path; 0,0
      | t::r -> path_ref := r; t in
    let w,h,p = surface_dims screen in
    let npos = {opos with x = (opos.x + dx) mod w; y = (opos.y + dy) mod h} in
    pos_ref := npos;
    write_message screen 200 600
      (Printf.sprintf "X = %d, Y = %d" npos.x npos.y);
    move_sprite screen opos npos sprite;
    ()
    
end
let event_log = false
let print_event s = 
  if not event_log then () else
  Printf.printf "Time %d\t:" (Sdltimer.get_ticks ());
  Printf.printf "%s" s; flush stdout


let () =
  Printf.printf "Alpha value is %d\n" (get_alpha screen);
  flush stdout

let () = set_alpha screen 1


let black = (0,0,0)
let white = (255,255,255)
let noop () =
  let s_text = Sdlttf.render_text font (Sdlttf.SOLID white) "SVEN" in
  blit_surface ~src:s_text ~dst:screen ();
  update_rect screen;
  ()


let () =
  Printf.printf "Alpha value is %d\n" (get_alpha screen);
  flush stdout

let info_test test = 
  let w,h,p = surface_dims test in
  Printf.printf "Surface size : w = %d, h = %d, p = %d\n" w h p;
  Printf.printf "Surface bpp  = %d\n" (surface_bpp test);
  (*
  Printf.printf "Surface rmask = %d\n" (surface_rmask test);
  Printf.printf "Surface gmask = %d\n" (surface_gmask test);
  Printf.printf "Surface bmask = %d\n" (surface_bmask test);
  Printf.printf "Surface amask = %d\n" (surface_amask test);
  *)
  flush stdout

let test = Sdlttf.render_text font (Sdlttf.SOLID white) "Hello"
let () = info_test test
(*
let test = surface_display_format test
let () = info_test test
*)
(*
let () = try font_stuff () with Sdlttf.SDLttf_exception s ->
  Printf.eprintf "Font exception raised : %s.\n" s; flush stderr
*)

(*
let keyboard_event_handler key state x y = 
  print_event "key event\n";
  if key = Sdlkey.KEY_ESCAPE then quit () else ()


let ref_pos = ref (0,0,1,1)
let mouse_event_handler button state x y = 
  print_event "mouse event\n";
  match button with 
    | Sdlmouse.BUTTON_LEFT -> begin
      match state with
        | BUTTON_STATE_RELEASED -> ()
        | BUTTON_STATE_PRESSED -> 
	  let screen_rect = let a,b,c,d = !ref_pos in Rect (a,b,c,d) in
	  ignore (surface_fill_rect screen screen_rect black);
          update_rect screen screen_rect;
          let t = surface_display_format (Sdlttf.render_text font "Hello"
            (rgb_vector_of_color white) (rgb_vector_of_color black)) in
	  let tv = surface_loadBMP "vaiseau.bmp" in
          let t_rect = surface_rect t in
          let w = surface_width t and h = surface_height t in
	  let x = x - w / 2 and y = y - h / 2 in
	  let mx = (surface_width screen) - w in
	  let my = (surface_height screen) - h in
	  let y = if y > my then my else if y < 0 then 0 else y in
	  let x = if x > mx then mx else if x < 0 then 0 else x in
	  let screen_rect = Rect(x,y,w,h) in
	  ref_pos := (x,y,w,h);
	  surface_blit t t_rect screen screen_rect;
          update_rect screen screen_rect
      end
    | Sdlmouse.BUTTON_MIDDLE -> ()
    | Sdlmouse.BUTTON_RIGHT -> ()

let mousemotion_event_handler x y = 
  print_event "mousemotion event\n"

let idle_event_handler () =
  Sprite.idle_loop screen
  (*
  print_event "idle event\n"
  *)

let () = set_keyboard_event_func keyboard_event_handler
let () = set_mouse_event_func mouse_event_handler
let () = set_mousemotion_event_func mousemotion_event_handler
let () = set_idle_event_func idle_event_handler

let () = start_event_loop ()
let () = Sdlttf.close_font font
let () = surface_free screen
*)
let () = Sdlmouse.show_cursor true
*)
(*
let s = "OCamlSDL" ;;
wm_set_caption s s;;

let clouds = surface_loadBMP "../images/clouds.bmp";;
let icon = surface_loadBMP "../images/icon.bmp";;
let logo = Sdlloader.load_image "../images/ocamlsdl.png";; 

let black = color_of_int(0,0,0) ;;
let white = color_of_int(255,255,255) ;;

let screen_fill img = 
  let screen = get_display_surface()
  in
    surface_blit img (surface_rect screen) screen (surface_rect screen);
    flip screen;;

let fill_with bg = 
  flip (surface_fill_rect screen (surface_rect screen) bg);;

let random_placement src dst =
  let h = surface_height src
  and w = surface_width src
  in
  let y_max = (surface_height dst) - h
  and x_max = (surface_width dst) - w
  in
  let r_smiley = (Rect(Random.int x_max, 
		       Random.int y_max,
		       w,h))
  and r_src = (surface_rect src)
    
  in
    surface_blit src r_src dst r_smiley;
    update_rect dst r_smiley;;
    
let place_max_icons max delay = 
  surface_set_colorkey icon (Some (Sdlvideo.IntColor(255, 255, 255)));
  for i = 0 to max do
    ignore (surface_set_alpha icon (Random.float 1.0));
    random_placement icon screen;
    Sdltimer.delay delay; 
  done;;

let display_text f s bg fg =
  let s_text = Sdlttf.render_text f s
	       (rgb_vector_of_color bg) 
	       (rgb_vector_of_color fg)
  in
    random_placement s_text screen ;;

let f = Sdlttf.open_font "../fonts/Arial.ttf" 20;;


 fill_with white;;
 display_text f "Wait 2 seconds..." black white ;;
 Sdltimer.delay 2000 ;;
 
 fill_with black;;
 display_text f "Going in the sky in 2 seconds and let's the pacmen" white black;;
 Sdltimer.delay 2000 ;;
 
 screen_fill clouds;;
 flip screen ;;
 (*  display_text f "Let's the pacmen !!" white black;; *)
 place_max_icons 100 1;;
 Sdltimer.delay 2000 ;;
 
 fill_with white;;
 screen_fill logo ;;
 flip screen;;
 display_text f ("Made with "^s)  black white;;
 Sdltimer.delay 5000 ;;
 
 Sdlttf.close_font f ;;



*)
